/*
 ATmega GPS SMS Alerter
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 9/10/2013
 Last Modified: 25/01/2014
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

#define MMA_MOSFET PC5
#define GPS_MOSFET PC4
#define NOKIA_MOSFET PC3
#define NOKIA_POWER_BUTTON PB2

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(uint8_t ii) {
	uint8_t bb;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = bb;
	WDTCSR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(int timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}

// ADC Read
int adc_read(uint8_t pin) {
	uint8_t low, high;
	
	ADMUX = (pin & 0x3f);
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// Setup the ATtiny
void setup(void) {
	UBRR0L = 155; // 9,600 Baud rate
	sbi(UCSR0A, U2X0); // Double rate
	sbi(UCSR0B, RXEN0); // Receiver enable
	
	// Set ADC prescale factor to 64
	// 12 MHz / 64 = 187.5 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS2);
	sbi(ADCSRA, ADPS1);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	// Setup outputs
	DDRC |= (1<<MMA_MOSFET);
	DDRC |= (1<<GPS_MOSFET);
	DDRC |= (1<<NOKIA_MOSFET);
	DDRB |= (1<<NOKIA_POWER_BUTTON);
	
	DDRB |= (1<<PB1);
	PORTB |= (1<<PB1);
	_delay_ms(500);
	PORTB &= ~(1<<PB1);
	
	sei(); // Turn on interrupts
}
